From b0a6fa16422c270709c05ae41fb1ac33d1e0c3e7 Mon Sep 17 00:00:00 2001
From: Peti Gomori <gomoripeti@gmail.com>
Date: Fri, 15 Jun 2012 20:26:33 +0200
Subject: [PATCH 1/2] [observer] Edit table row in a multiline text dialog

---
 lib/observer/src/observer_lib.erl      |   54 +++++++++++++++++++++++++++++++-
 lib/observer/src/observer_tv_table.erl |    6 ++--
 2 files changed, 57 insertions(+), 3 deletions(-)

diff --git a/lib/observer/src/observer_lib.erl b/lib/observer/src/observer_lib.erl
index 3b924d4..4077f83 100644
--- a/lib/observer/src/observer_lib.erl
+++ b/lib/observer/src/observer_lib.erl
@@ -19,7 +19,7 @@
 -module(observer_lib).
 
 -export([get_wx_parent/1,
-	 display_info_dialog/1, user_term/3,
+	 display_info_dialog/1, user_term/3, user_term_multiline/3,
 	 interval_dialog/4, start_timer/1, stop_timer/1,
 	 display_info/2, fill_info/2, update_info/2, to_str/1,
 	 create_menus/3, create_menu_item/3,
@@ -347,6 +347,58 @@ user_term(Parent, Title, Default) ->
 	    cancel
     end.
 
+user_term_multiline(Parent, Title, Default) ->
+    Dialog = wxDialog:new(Parent, ?wxID_ANY, Title,
+			  [{style, ?wxDEFAULT_DIALOG_STYLE bor
+			    ?wxRESIZE_BORDER}]),
+    Panel = wxPanel:new(Dialog),
+
+    TextCtrl = wxTextCtrl:new(Panel, ?wxID_ANY,
+			      [{value, Default},
+			       {style, ?wxDEFAULT bor ?wxTE_MULTILINE}]),
+    Line = wxStaticLine:new(Panel, [{style, ?wxLI_HORIZONTAL}]),
+
+    Buttons = wxDialog:createButtonSizer(Dialog, ?wxOK bor ?wxCANCEL),
+
+    InnerSizer = wxBoxSizer:new(?wxVERTICAL),
+    wxSizer:add(InnerSizer, TextCtrl,
+		[{flag, ?wxEXPAND bor ?wxALL},{proportion, 1},{border, 5}]),
+    wxSizer:add(InnerSizer, Line,
+		[{flag, ?wxEXPAND},{proportion, 0},{border, 5}]),
+    wxPanel:setSizer(Panel, InnerSizer),
+
+    TopSizer = wxBoxSizer:new(?wxVERTICAL),
+    wxSizer:add(TopSizer, Panel,
+		[{flag, ?wxEXPAND bor ?wxALL},{proportion, 1},{border, 5}]),
+    wxSizer:add(TopSizer, Buttons,
+		[{flag, ?wxEXPAND bor ?wxBOTTOM bor ?wxRIGHT},{border, 10}]),
+
+    % calculate the size of TopSizer when the whole user_term
+    % fits in the TextCtrl
+    DC = wxClientDC:new(Panel),
+    W = wxDC:getCharWidth(DC),
+    H = wxDC:getCharHeight(DC),
+    {EW, EH} = wxDC:getMultiLineTextExtent(DC, Default),
+    wxSizer:setItemMinSize(InnerSizer, 0, EW+2*W, EH+H),
+    TopSize = wxSizer:getMinSize(TopSizer),
+    % reset min size of TextCtrl to 40 chararacters * 4 lines
+    wxSizer:setItemMinSize(InnerSizer, 0, 40*W, 4*H),
+
+    wxWindow:setSizerAndFit(Dialog, TopSizer),
+    wxSizer:setSizeHints(TopSizer, Dialog),
+
+    wxWindow:setClientSize(Dialog, TopSize),
+
+    case wxDialog:showModal(Dialog) of
+	?wxID_OK ->
+	    Str = wxTextCtrl:getValue(TextCtrl),
+	    wxDialog:destroy(Dialog),
+	    parse_string(ensure_last_is_dot(Str));
+	?wxID_CANCEL ->
+	    wxDialog:destroy(Dialog),
+	    cancel
+    end.
+
 parse_string(Str) ->
     try
 	Tokens = case erl_scan:string(Str) of
diff --git a/lib/observer/src/observer_tv_table.erl b/lib/observer/src/observer_tv_table.erl
index 3930f9e..231a4c3 100644
--- a/lib/observer/src/observer_tv_table.erl
+++ b/lib/observer/src/observer_tv_table.erl
@@ -218,8 +218,8 @@ search_area(Parent) ->
 	    search=TC1,goto=TC2,radio={Nbtn,Pbtn,Cbtn}}.
 
 edit(Index, #state{pid=Pid, frame=Frame}) ->
-    Str = get_row(Pid, Index, all),
-    case observer_lib:user_term(Frame, "Edit object:", Str) of
+    Str = get_row(Pid, Index, all_multiline),
+    case observer_lib:user_term_multiline(Frame, "Edit object:", Str) of
 	cancel -> ok;
 	{ok, Term} -> Pid ! {edit, Index, Term};
 	Err = {error, _} -> self() ! Err
@@ -628,6 +628,8 @@ get_row(From, Row, Col, Table) ->
     case lists:nth(Row+1, Table) of
 	[Object|_] when Col =:= all ->
 	    From ! {self(), format(Object)};
+	[Object|_] when Col =:= all_multiline ->
+	    From ! {self(), io_lib:format("~p", [Object])};
 	[Object|_] when tuple_size(Object) >= Col ->
 	    From ! {self(), format(element(Col, Object))};
 	_ ->
-- 
1.7.10.4

